{################################################################ LIBSIO.INC }
{ Purpose...: Serial I/O using SIO A, include file
{ DependsOn.: LIB.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 22.01.2023
{############################################################################}

{******************************************** constant and type definitions }
CONST g_bLibSioNOK:     BYTE = 0;                { operation not complete   }
CONST g_bLibSioOK:      BYTE = 1;                { operation completed, ok  }
CONST g_bLibSioTIMEOUT: BYTE = 2;                { operation timed out      }
CONST g_bLibSioCANCEL:  BYTE = 3;                { operation cancelled (esc)}

{------------------------------------------------ hardware handshake signals }
CONST g_bLibSioTX_HS_RTS = $02;
CONST g_bLibSioTX_HS_DTR = $80;
CONST g_bLibSioTX_HS_OFF = $00;
CONST g_bLibSioTX_HS_ON  = $82;

{------------------------------------------------------- SIO port addresses }
CONST m_bPORT_CTRL:     BYTE = 6;                { SIO control port         }
CONST m_bPORT_DATA:     BYTE = 4;                { SIO data port            }
CONST m_bPORT_BAUD:     BYTE = 0;                { SIO baudrate port        }

{---------------------------------------------------------------- baudrates }
CONST m_bBAUD_110:  BYTE = 2;
CONST m_bBAUD_300:  BYTE = 5;
CONST m_bBAUD_600:  BYTE = 6;
CONST m_bBAUD_1200: BYTE = 7;
CONST m_bBAUD_2400: BYTE = 10;
CONST m_bBAUD_4800: BYTE = 12;
CONST m_bBAUD_9600: BYTE = 14;
CONST m_bBAUD_19K2: BYTE = 15;

{-------------------------- parity define values to OR into write register 4 }
CONST m_bPARITY_NONE = $00;
CONST m_bPARITY_EVEN = $03;
CONST m_bPARITY_ODD  = $01;

{------------------------ stopbits define values to OR into write register 4 }
CONST m_bSTOPBITS_1   = $04;
CONST m_bSTOPBITS_1P5 = $08;
CONST m_bSTOPBITS_2   = $0C;

{------------------------ databits define values to OR into write register 3 }
CONST m_bDATABITS_RX_5 = $00;
CONST m_bDATABITS_RX_6 = $80;
CONST m_bDATABITS_RX_7 = $40;
CONST m_bDATABITS_RX_8 = $C0;

CONST m_bDATABITS_TX_5 = $00;
CONST m_bDATABITS_TX_6 = $40;
CONST m_bDATABITS_TX_7 = $20;
CONST m_bDATABITS_TX_8 = $60;

CONST m_bTX_TIMEOUT: BYTE = 200;                 { SIO-Tx-ready timeout      }

VAR m_iDelayChar: INTEGER;                       { delay between characters  }
VAR m_iDelayLine: INTEGER;                       { delay between lines       }
VAR m_bSioWr5:    BYTE;                          { static contents of WR5    }

{************************************************************ implementation }
{============================================================= LibSio_Init() }
{ Purpose...: Initialize SIO
{ Input.....: i_fInit.....: TRUE= initialize, FALSE= deinitialize
{             i_iBaudRate.: Baudrate to use (110..19200)
{             i_cParity...: Parity to use, 'N', 'O' or 'E'
{             i_bDataBits.: Number of databits, 7 or 8
{             i_bStopBits.: Number od stopbits, 1 or 2
{             i_iDelayChar: Delay to insert between chars on send
{             i_iDelayLine: Delay to insert between lines on send
{============================================================================}
PROCEDURE LibSio_Init(i_fInit: BOOLEAN; i_iBaudRate: INTEGER;
                      i_cParity: CHAR; i_bDataBits: BYTE; i_bStopBits: BYTE;
                      i_iDelayChar: INTEGER; i_iDelayLine: INTEGER);

  VAR bBaud    : BYTE;
  VAR bWr3     : BYTE;
  VAR bWr4     : BYTE;
  VAR bWr5     : BYTE;
  VAR fTxEmpty : BOOLEAN;

BEGIN
  m_iDelayChar := i_iDelayChar;
  m_iDelayLine := i_iDelayLine;

  bBaud := 0;
  bWr3 := 0;
  bWr4 := 0;
  bWr5 := 0;

  {------------------------------------------------- create baudrate divisor }
  CASE i_iBaudRate OF
      110: bBaud := m_bBAUD_110;
      300: bBaud := m_bBAUD_300;
      600: bBaud := m_bBAUD_600;
     1200: bBaud := m_bBAUD_1200;
     2400: bBaud := m_bBAUD_2400;
     4800: bBaud := m_bBAUD_4800;
     9600: bBaud := m_bBAUD_9600;
    19200: bBaud := m_bBAUD_19K2;
  END;

  CASE i_cParity OF
    'N': bWr4 := m_bPARITY_NONE;
    'E': bWr4 := m_bPARITY_EVEN;
    'O': bWr4 := m_bPARITY_ODD;
  END;

  CASE i_bStopBits OF
    1: bWr4 := (bWr4 OR m_bSTOPBITS_1);
    2: bWr4 := (bWr4 OR m_bSTOPBITS_2);
  END;

  CASE i_bDataBits OF
    5: BEGIN
         bWr3 := m_bDATABITS_RX_5;
         bWr5 := m_bDATABITS_TX_5;
       END;
    6: BEGIN
         bWr3 := m_bDATABITS_RX_6;
         bWr5 := m_bDATABITS_TX_6;
       END;
    7: BEGIN
         bWr3 := m_bDATABITS_RX_7;
         bWr5 := m_bDATABITS_TX_7;
       END;
    8: BEGIN
         bWr3 := m_bDATABITS_RX_8;
         bWr5 := m_bDATABITS_TX_8;
       END;
  END;

  {------------------------------------------------ on init set fixed params }
  IF (i_fInit = TRUE) THEN BEGIN
    bWr3 := (bWr3 OR $01);                       { set RX enable             }
    bWr4 := (bWr4 OR $40);                       { set X16 clock mode        }
    bWr5 := (bWr5 OR $8A);                       { set DTR, RTS, TX enable   }
  END

  {----------------------------------------------------------------- de-init }
  ELSE BEGIN
    bBaud := m_bBAUD_9600;
    bWr3 := (bWr3 AND $FE);
    bWr5 := $64;

    {-------------------------------------------- wait until tx buffer empty }
    WHILE ((PORT[m_bPORT_CTRL] AND $04) = $00) DO BEGIN END;
    fTxEmpty := FALSE;

    WHILE (fTxEmpty = FALSE) DO BEGIN
      PORT[m_bPORT_CTRL] := $01;
      fTxEmpty := ((PORT[m_bPORT_CTRL] AND $01) = $01);
    END;
  END;

  {---------------------------------------------------------- initialize SIO }
  PORT[m_bPORT_BAUD] := bBaud;                   { set baudrate              }
  PORT[m_bPORT_CTRL] := $18;                     { SIO reset                 }
  PORT[m_bPORT_CTRL] := $04;                     { select WR4                }
  PORT[m_bPORT_CTRL] := bWr4;                    { stop bits, parity, X16clk }
  PORT[m_bPORT_CTRL] := $01;                     { select WR1                }
  PORT[m_bPORT_CTRL] := $00;                     { no interrupts             }
  PORT[m_bPORT_CTRL] := $05;                     { select WR5                }
  PORT[m_bPORT_CTRL] := bWr5;                    { databits, set DTR&RTS     }
  PORT[m_bPORT_CTRL] := $03;                     { select WR3                }
  PORT[m_bPORT_CTRL] := bWr3;                    { databits, enable RCVR     }

  m_bSioWr5 := bWr5;                             { save for RTS/DTR changes  }
END;
{========================================================= eoF LibSio_Init() }

{========================================================== LibSio_SetTxHs() }
{ Purpose...: Set TX handshake signals RTS and/or DTR
{ Input.....: i_bTxHs: RTS/DTR status to set. Combination of m_bTX_HS_XXX
{============================================================================}
PROCEDURE LibSio_SetTxHs(i_bTxHs: BYTE);

  VAR bWr5: BYTE;

BEGIN
  bWr5 := m_bSioWr5;
  bWr5 := (bWr5 AND (NOT g_bLibSioTX_HS_ON));
  bWr5 := (bWr5 OR i_bTxHs);

  PORT[m_bPORT_CTRL] := $05;                     { select WR5                }
  PORT[m_bPORT_CTRL] := bWr5;                    { databits, set DTR&RTS     }
END;
{====================================================== eoP LibSio_SetTxHs() }

{========================================================== LibSio_RxClear() }
{ Purpose...: Clear SIO's rx buffer
{============================================================================}
PROCEDURE LibSio_RxClear;

  VAR bDummy: BYTE;

BEGIN

  WHILE ((PORT[m_bPORT_CTRL] AND $01) = $01) DO BEGIN
    bDummy := PORT[m_bPORT_DATA];
  END;
END;
{====================================================== eoP LibSio_RxClear() }

{=========================================================== LibSio_RxByte() }
{ Purpose...: Receive one byte over serial link
{             Can be interrupted with <esc>
{ Input.....: i_pbRx...: Where to store received byte, Can be NIL for dummy
{                        reads.
{             i_iWait..: Wait time, 0= return immediately, -1= wait forever
{             i_fKbdBrk: TRUE= check for <esc> from keyboard, FALSE= do not
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION LibSio_RxByte(i_pbRx: g_tpByte; i_iWait: INTEGER;
                       i_fKbdBrk: BOOLEAN): BYTE;

  VAR bFnResult : BYTE;
  VAR iTimer    : INTEGER;
  VAR bSioStat  : BYTE;
  VAR cKey      : CHAR;
  VAR bRx       : BYTE;

BEGIN
  bFnResult := g_bLibSioNOK;
  iTimer := i_iWait;

  {----------------------------------- wait until character ready or timeout }
  WHILE (((PORT[m_bPORT_CTRL] AND $01) = $00) AND (iTimer <> 0)) DO BEGIN

    IF (i_iWait <> -1) THEN BEGIN
      iTimer := iTimer - 1;

      IF (iTimer = 0) THEN BEGIN
        bFnResult := g_bLibSioTIMEOUT;
      END;
    END;

    IF (i_fKbdBrk = TRUE) THEN BEGIN

      IF (Lib_GetKey(g_cESC, FALSE) <> CHR(0)) THEN BEGIN
        iTimer := 0;
        bFnResult := g_bLibSioCANCEL;
      END;
    END;
  END;

  {---------------------------------------------------------- read port data }
  IF (PORT[m_bPORT_CTRL] AND $01 = $01) THEN BEGIN
    bRx := PORT[m_bPORT_DATA];

    IF (i_pbRx <> NIL) THEN BEGIN
     i_pbRx^ := bRx;
    END;

    bFnResult := g_bLibSioOK;
  END;

  {------ check keyboard cancel request (in case WHILE loop did not trigger) }
  IF (i_fKbdBrk = TRUE) THEN BEGIN

    IF (Lib_GetKey(g_cESC, FALSE) <> CHR(0)) THEN BEGIN
      bFnResult := g_bLibSioCANCEL;
    END;
  END;

  LibSio_RxByte := bFnResult;
END;
{======================================================= eoF LibSio_RxByte() }

{========================================================= LibSio_RxString() }
{ Purpose...: Receive string over serial link
{ Input.....: i_pbBuf....: Address of buffer to write data to
{             i_pbRxBytes: Address to store number of received bytes
{             i_iEnd.....: At which byte to stop reception, -1= not used
{             i_iTimeout.: Stop after time with no activity
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION LibSio_RxString(i_pbBuf: g_tpByte; i_pbRxBytes: g_tpByte;
                         i_iEnd: INTEGER; i_iTimeout: INTEGER): BYTE;

  VAR bSio     : BYTE;
  VAR bRxBytes : BYTE;
  VAR bEnd     : BYTE;
  VAR fDone    : BOOLEAN;

BEGIN
  fDone := FALSE;
  bSio := g_bLibSioNOK;
  bEnd := LO(i_iEnd);
  bRxBytes := 0;

  {--------------------- loop until line complete or keyboard cancel request }
  WHILE (fDone = FALSE) DO BEGIN
    bSio := LibSio_RxByte(i_pbBuf, i_iTimeout, TRUE);

    {--------------------------- byte received, inc pointer and byte counter }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      bRxBytes := bRxBytes + 1;
    END;

    {--------------------------------------- check for timeout or user abort }
    IF ((bSio = g_bLibSioCANCEL) OR (bSio = g_bLibSioTIMEOUT)) THEN BEGIN
      fDone := TRUE;
    END;
      
    {-------------------------------------------- check for termination byte }
    IF ((bSio = g_bLibSioOK) AND (i_iEnd <> -1)) THEN BEGIN
      IF (i_pbBuf^ = bEnd) THEN BEGIN
        fDone := TRUE;
      END;
    END;

    i_pbBuf := PTR(ORD(i_pbBuf) + 1);
  END;

  IF (i_pbRxBytes <> NIL) THEN BEGIN
    i_pbRxBytes^ := bRxBytes;
  END;

  LibSio_RxString := bSio;
END;
{===================================================== eoF LibSio_RxString() }

{=========================================================== LibSio_TxByte() }
{ Purpose...: Send one byte over serial link
{ Input.....: i_bTx: byte to send
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION LibSio_TxByte(i_bTx: BYTE): BYTE;

  VAR bSio     : BYTE;
  VAR bTimer   : BYTE;
  VAR bSioStat : BYTE;
  VAR cKbd     : CHAR;

BEGIN
  bSio := g_bLibSioNOK;
  bTimer := m_bTX_TIMEOUT;

  {--------------------------------- wait until tx register empty or timeout }
  WHILE (((PORT[m_bPORT_CTRL] AND $04) = $00) AND (bTimer > 0)) DO BEGIN
    bTimer := bTimer - 1;
  END;

  {---------------------------------------- get byte from SIO if not timeout }
  IF (bTimer > 0) THEN BEGIN
    PORT[m_bPORT_DATA] := i_bTx;
    bSio := g_bLibSioOK;
  END ELSE BEGIN
    bSio := g_bLibSioTIMEOUT;
  END;

  {--------------------------------------- check for keyboard cancel request }
  IF (Lib_GetKey(g_cESC, FALSE) <> CHR(0)) THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END;

  {---------------------------- apply inter-character delay if byte received }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    Lib_Delay(m_iDelayChar);
  END;

  LibSio_TxByte := bSio;
END;
{======================================================= eoF LibSio_TxByte() }

{========================================================= LibSio_TxString() }
{ Purpose...: Send string over serial link
{ Input.....: i_stTx..: String to send
{             i_fWrite: TRUE= write each char to screen, FALSE= be silent
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION LibSio_TxString(i_stTx: g_tSt; i_fWrite: BOOLEAN): BYTE;

  VAR bLen : BYTE;
  VAR bNdx : BYTE;
  VAR bSio : BYTE;
  VAR bTx  : BYTE;

BEGIN
  bSio := g_bLibSioNOK;
  bNdx := 1;
  bLen := LENGTH(i_stTx);

  {------------------------- send line until done or keyboard cancel request }
  WHILE ((bNdx <= bLen) AND (bSio = g_bLibSioNOK)) DO BEGIN
    bTx := ORD(i_stTx[bNdx]);
    bSio := LibSio_TxByte(bTx);

    IF (bSio = g_bLibSioOK) THEN BEGIN
      bNdx := bNdx + 1;

      IF ((i_fWrite = TRUE) AND (bTx <> 0)) THEN BEGIN
        WRITE(CHR(bTx));
      END;

      IF (bNdx < (bLen + 1)) THEN BEGIN
        bSio := g_bLibSioNOK;
      END;
    END;
  END;

  {------------------------------------------------------ perform line delay }
  Lib_Delay(m_iDelayLine);

  LibSio_TxString := bSio;
END;
{===================================================== eoF LibSio_TxString() }

{=========================================================== LibSio_TxWait() }
{ Purpose...: Send string over serial link, wait for reception of a string
{ Input.....: i_stTx..: String to send
{             i_stRx..: String to wait for after i_stTx is received. 'x' in
{                       string are dont' care, a byte must be just received
{             i_iTmo..: Timeout value, -1 = wait infinite
{             i_fWrite: TRUE= write received chars to screen, FALSE= silent
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION LibSio_TxWait(i_stTx: g_tst; i_stRx: g_tst;
                       i_iTmo: INTEGER; i_fWrite: BOOLEAN): BYTE;

  VAR bLen : BYTE;
  VAR bNdx : BYTE;
  VAR bSio : BYTE;
  VAR bTx  : BYTE;
  VAR bRx  : BYTE;
  VAR pbRx : ^BYTE;

BEGIN
  bSio := g_bLibSioOK;
  bLen := LENGTH(i_stTx);
  bNdx := 1;
  pbRx := PTR(ADDR(bRx));

  {------------------------------------------ send string, wait for same echo }
  WHILE ((bNdx <= bLen) AND (bSio = g_bLibSioOK)) DO BEGIN
    bTx := ORD(i_stTx[bNdx]);
    bSio := LibSio_TxByte(bTx);

    {---------------------------------------------------------- wait for echo }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      bSio := LibSio_RxByte(pbRx, -1, TRUE);
    END;

    {-------------------------------------------------- check if same as sent }
    IF ((bSio = g_bLibSioOK) AND (bRx <> 0)) THEN BEGIN
      bNdx := bNdx + 1;

      IF (bRx <> bTx) THEN BEGIN
        bSio := g_bLibSioNOK;
      END;

      IF (bSio = g_bLibSioOK) THEN BEGIN
        IF ((i_fWrite = TRUE) AND (bRx <> $7F)) THEN BEGIN
          WRITE(CHR(bRx));
        END;
      END;
    END;
  END;

  bLen := LENGTH(i_stRx);
  bNdx := 1;

  {---------------------------------------------- receive and check response }
  WHILE ((bNdx <= bLen) AND (bSio = g_bLibSioOK)) DO BEGIN
    bTx := ORD(i_stRx[bNdx]);
    bSio := LibSio_RxByte(pbRx, i_iTmo, TRUE);

    IF ((bSio = g_bLibSioOK) AND (bRx <> 0)) THEN BEGIN
      bNdx := bNdx + 1;

      IF (bTx <> ORD('x')) THEN BEGIN
        IF (bTx <> bRx) THEN BEGIN
          bSio := g_bLibSioNOK;
        END;
      END;

      IF (bSio = g_bLibSioOK) THEN BEGIN
        IF ((i_fWrite = TRUE) AND (bRx <> $7F)) THEN BEGIN
          WRITE(CHR(bRx));
        END;
      END;
    END;
  END;

  LibSio_TxWait := bSio;
END;
{======================================================= eoF LibSio_TxWait() }

{############################################################ EOF LIBSIO.INC }