{################################################################ OSIASM.INC }
{ Purpose...: OSI include file: Send/receive ASM files
{ DependsOn.: LIB.INC, LIBSIO.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 29.01.2023
{############################################################################}

{*************************************************************** definitions }
CONST m_iASM_BAUDRATE:   INTEGER  = 300;         { baudrate for .ASM files   }
CONST m_iASM_TXDLY_CHAR: INTEGER  = 0;           { tx delay between chars    }
CONST m_iASM_TXDLY_LINE: INTEGER  = 5000;        { tx delay detween lines    }

{************************************************************ implementation }
{============================================================= Asm_InitSio() }
{ Purpose...: Init/deinit SIO
{ Input.....: i_fInit: TRUE= init, FALSE= de-init
{============================================================================}
PROCEDURE Asm_InitSio(i_fInit: BOOLEAN);

BEGIN

  IF (i_fInit = TRUE) THEN BEGIN

    LibSio_Init(TRUE, m_iASM_BAUDRATE, 'N', 8, 1,
                m_iASM_TXDLY_CHAR, m_iASM_TXDLY_LINE);

  END ELSE BEGIN
    LibSio_Init(FALSE, 0, 'N', 0, 0, 0, 0);
  END;
END;
{========================================================= eoF Asm_InitSio() }

{============================================================ Asm_SendFile() }
{ Purpose...: Send .ASM file to SIO
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Asm_SendFile: BYTE;

  VAR bSio       : BYTE;
  VAR pstLine    : g_tpStLine;
  VAR bNdx       : BYTE;
  VAR bLen       : BYTE;
  VAR fLineValid : BOOLEAN;
  VAR cKbd       : CHAR;
  VAR fStart     : BOOLEAN;

BEGIN
  bSio := g_bLibSioOK;

  {------------------------------ display OSI hint and wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_TRANSMIT, g_stFile, ' (', File_GetBufLines, ' lines)');
  WRITE('InAsmEdit type `L`<CR>.', g_stMSG_CONTINUE);
  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_TRANSMITTING, g_stFile, g_stMSG_CANCEL);
  END;

  {---------------------------------------------------------------- init SIO }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    Asm_InitSio(TRUE);
  END;

  pstLine := PTR(1);
  fStart := TRUE;

  {----------------------------------------------------------- send the file }
  WHILE ((pstLine <> NIL) AND (bSio = g_bLibSioOK)) DO BEGIN
    pstLine := File_GetString(-1, fStart, NIL);
    fStart := FALSE;

    IF (pstLine <> NIL) THEN BEGIN
      bLen := LENGTH(pstLine^);
      fLineValid := FALSE;
      bNdx := 1;

      {----------------------------- only send lines beginning with a number }
      WHILE ((fLineValid = FALSE) AND (bNdx <= bLen)) DO BEGIN
        fLineValid := ((pstLine^[bNdx] >= '0') AND (pstLine^[bNdx] <= '9'));
        IF (fLineValid = FALSE) THEN bNdx := bNdx + 1;
      END;

      {------------------------------------ write the line to screen and SIO }
      IF (fLineValid = TRUE) THEN BEGIN
        WRITELN(COPY(pstLine^, bNdx, bLen));
        bSio := LibSio_TxString(pstLine^ + g_stCRLF, FALSE);
      END;
    END;
  END;

  WRITELN;

  {--------------------------------------------------------------- cancelled }
  IF (bSio = g_bLibSioCANCEL) THEN BEGIN
    WRITE('Cancelled. ');
  END;

  WRITELN('Press <SPACE><CR> on OSI to end LOAD');

  Asm_InitSio(FALSE);
  Asm_SendFile := bSio;
END;
{======================================================== eoP Asm_SendFile() }

{========================================================= Asm_ReceiveFile() }
{ Purpose...: Receive ASM file
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Asm_ReceiveFile: BYTE;

  CONST iEND_TIMEOUT : INTEGER = 1500;

  VAR bSio     : BYTE;
  VAR cKbd     : CHAR;
  VAR cRx      : CHAR;
  VAR pbRx     : ^BYTE;
  VAR iTimeout : INTEGER;
  VAR fDone    : BOOLEAN;

BEGIN
  fDone := FALSE;
  bSio := g_bLibSioOK;
  iTimeout := -1;
  pbRx := PTR(ADDR(cRx));

  {----------------------------- display OSI hint and wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_RECEIVE, g_stFile, ' (', File_GetBufSize, ' bytes free)');
  WRITE('InAsmEdit type `S`<CR>`P`, then <CR> after continued.');
  WRITE(g_stMSG_CONTINUE);

  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_RECEIVING, g_stFile, g_stMSG_CANCEL);
    LibSio_RxClear;
  END;

  {---------------------------------------------------------------- init SIO }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    Asm_InitSio(TRUE);
  END;

  {-------------------------------------------- loop until done or cancelled }
  WHILE ((bSio = g_bLibSioOK) AND (fDone = FALSE)) DO BEGIN
    bSio := LibSio_RxByte(pbRx, iTimeout, TRUE);

    {-------------------------------------------- one character was received }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      iTimeout := iEND_TIMEOUT;

      IF (File_PutBuffer(cRx) = FALSE) THEN BEGIN
        bSio := g_bLibSioCANCEL;
        cRx := CHR(0);
      END;

      {------------------------------------------- write character to screen }
      IF (ORD(cRx) <> 0) THEN BEGIN
        WRITE(cRx);
      END;
    END;

    IF ((iTimeout <> -1) AND (bSio = g_bLibSioTIMEOUT)) THEN BEGIN
      fDone := TRUE
    END;
  END;

  {----------------------------------------------------- save buffer to disk }
  IF ((fDone = TRUE) AND (bSio = g_bLibSioTIMEOUT)) THEN BEGIN
    File_WriteBuffer;
    bSio := g_bLibSioOK;
  END;

  {--------------------------------------------------------------- cancelled }
  IF (bSio = g_bLibSioCANCEL) THEN BEGIN
    WRITELN;
    WRITE('Cancelled. ');

    IF (iTimeout <> -1) THEN BEGIN
      WRITE('Write buffer to ', g_stFile, ' ? (Y/N) ');
      cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));

      IF (cKbd = 'Y') THEN BEGIN
        File_WriteBuffer;
      END;
    END;
  END;

  WRITELN;
  WRITELN('Type `L`<CR><SPACE><CR> on OSI to end SAVE');

  Asm_InitSio(FALSE);
  Asm_ReceiveFile := bSio;
END;
{===================================================== eoF Asm_ReceiveFile() }

{############################################################ EOF OSIASM.INC }
