{################################################################ OSIBAS.INC }
{ Purpose...: OSI include file: Send/receive BAS files
{ DependsOn.: LIB.INC, LIBSIO.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 30.08.2022, 29.01.2023
{############################################################################}

{*************************************************************** definitions }
CONST m_iBAS_BAUDRATE:   INTEGER  = 300;         { baudrate for .BAS files   }
CONST m_iBAS_TXDLY_CHAR: INTEGER  = 0;           { tx delay between chars    }

{---------------------------- 20220824: changed line delay from 1000 to 5000 }
{ some complex instructions (in brickout.bas) did not load correctly, some   }
{ lines were exchanged/forgotten or garbled                                  }
{----------------------------------------------------------------------------}
CONST m_iBAS_TXDLY_LINE: INTEGER  = 5000;        { tx delay detween lines    }

{************************************************************ implementation }
{============================================================= Bas_InitSio() }
{ Purpose...: Init/deinit SIO
{ Input.....: i_fInit: TRUE= init, FALSE= de-init
{============================================================================}
PROCEDURE Bas_InitSio(i_fInit: BOOLEAN);

BEGIN

  IF (i_fInit = TRUE) THEN BEGIN

    LibSio_Init(TRUE, m_iBAS_BAUDRATE, 'N', 8, 1, 
                m_iBAS_TXDLY_CHAR, m_iBAS_TXDLY_LINE);

  END ELSE BEGIN
    LibSio_Init(FALSE, 0, 'N', 0, 0, 0, 0);
  END;
END;
{========================================================= eoF Bas_InitSio() }

{============================================================ Bas_SendFile() }
{ Purpose...: Send .BAS file to SIO
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Bas_SendFile: BYTE;

  VAR bSio       : BYTE;
  VAR pstLine    : g_tpStLine;
  VAR iBufLine   : INTEGER;
  VAR bNdx       : BYTE;
  VAR bLen       : BYTE;
  VAR fLineValid : BOOLEAN;
  VAR cKbd       : CHAR;
  VAR fStart     : BOOLEAN;

BEGIN
  bSio := g_bLibSioOK;

  {------------------------------ display OSI hint and wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_TRANSMIT, g_stFile, ' (', File_GetBufLines, ' lines)');
  WRITE('In Basic type `LOAD`<CR>.', g_stMSG_CONTINUE);
  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_TRANSMITTING, g_stFile, g_stMSG_CANCEL);
  END;

  {---------------------------------------------------------------- init SIO }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    Bas_InitSio(TRUE);
  END;

  pstLine := PTR(1);
  fStart := TRUE;
  iBufLine := 0;

  {----------------------------------------------------------- send the file }
  WHILE ((pstLine <> NIL) AND (bSio = g_bLibSioOK)) DO BEGIN
    pstLine := File_GetString(iBufLine, fStart, NIL);
    fStart := FALSE;
    iBufLine := -1;

    {--------------------------------------------- send and display one line }
    IF (pstLine <> NIL) THEN BEGIN
      bLen := LENGTH(pstLine^);
      fLineValid := FALSE;
      bNdx := 1;

      {----------------------------- only send lines beginning with a number }
      WHILE ((fLineValid = FALSE) AND (bNdx <= bLen)) DO BEGIN
        fLineValid := ((pstLine^[bNdx] >= '0') AND (pstLine^[bNdx] <= '9'));
        IF (fLineValid = FALSE) THEN bNdx := bNdx + 1;
      END;

      {------------------------------------ write the line to screen and SIO }
      IF (fLineValid = TRUE) THEN BEGIN
        WRITELN(COPY(pstLine^, bNdx, bLen));
        bSio := LibSio_TxString(pstLine^ + g_stCRLF, FALSE);
      END;
    END;
  END;

  WRITELN;

  IF (bSio = g_bLibSioCANCEL) THEN BEGIN
    WRITE('Cancelled. ');
  END;

  WRITELN('Press <SPACE> on OSI to end LOAD');

  Bas_InitSio(FALSE);
  Bas_SendFile := bSio;
END;
{======================================================== eoP Bas_SendFile() }

{========================================================= Bas_ReceiveFile() }
{ Purpose...: Receive BAS file
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Bas_ReceiveFile: BYTE;

  CONST iEND_TIMEOUT : INTEGER = 1500;

  VAR bSio     : BYTE;
  VAR cKbd     : CHAR;
  VAR cRx      : CHAR;
  VAR pbRx     : ^BYTE;
  VAR iTimeout : INTEGER;

BEGIN
  bSio := g_bLibSioNOK;
  iTimeout := -1;
  pbRx := PTR(ADDR(cRx));

  {----------------------------- display OSI hint and wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_RECEIVE, g_stFile, ' (', File_GetBufSize, ' bytes free)');
  WRITE('In Basic type `SAVE`<CR>`LIST`, then <CR> after continued. ');
  WRITE(g_stMSG_CONTINUE);

  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_RECEIVING, g_stFile, g_stMSG_CANCEL);
    LibSio_RxClear;
  END;

  {---------------------------------------------------------------- init SIO }
  IF (bSio = g_bLibSioNOK) THEN BEGIN
    Bas_InitSio(TRUE);
  END;

  {-------------------------------------------- loop until done or cancelled }
  WHILE (bSio = g_bLibSioNOK) DO BEGIN
    bSio := LibSio_RxByte(pbRx, iTimeout, TRUE);

    {-------------------------------------------- one character was received }
    IF (bSio = g_bLibSioOK) THEN BEGIN
      bSio := g_bLibSioNOK;
      iTimeout := iEND_TIMEOUT;

      IF (File_PutBuffer(cRx) = FALSE) THEN BEGIN
        bSio := g_bLibSioCANCEL;
        cRx := CHR(0);
      END;

      {------------------------------------------- write character to screen }
      IF (ORD(cRx) <> 0) THEN BEGIN
        WRITE(cRx);
      END;
    END;
  END;

  {----------------------------------------------------- save buffer to disk }
  IF (bSio = g_bLibSioTIMEOUT) THEN BEGIN
    File_WriteBuffer;
    bSio := g_bLibSioOK;
  END;

  {--------------------------------------------------------------- cancelled }
  IF (bSio = g_bLibSioCANCEL) THEN BEGIN
    WRITELN;
    WRITE('Cancelled. Write buffer to ', g_stFile, ' ? (Y/N) ');
    cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));

    IF (cKbd = 'Y') THEN BEGIN
      File_WriteBuffer;
    END;
  END;

  WRITELN;
  WRITELN('Type `LOAD`<CR><SPACE><CR> to end SAVE');

  Bas_InitSio(FALSE);
  Bas_ReceiveFile := bSio;
END;
{===================================================== eoF Bas_ReceiveFile() }

{############################################################ EOF OSIBAS.INC }