{################################################################ OSILOD.INC }
{ Purpose...: OSI include file: Send/receive LOD files
{ DependsOn.: LIB.INC, LIBSIO.INC, OSIFILE.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 29.01.2023
{############################################################################}

{*************************************************************** definitions }
CONST m_iLOD_BAUDRATE:   INTEGER  = 300;         { baudrate for .LOD files   }
CONST m_iLOD_TXDLY_CHAR: INTEGER  = 0;           { tx delay between chars    }
CONST m_iLOD_TXDLY_LINE: INTEGER  = 0;           { tx delay between lines    }

{************************************************************ implementation }
{============================================================= Lod_InitSio() }
{ Purpose...: Init/deinit SIO
{ Input.....: i_fInit: TRUE= init, FALSE= de-init
{============================================================================}
PROCEDURE Lod_InitSio(i_fInit: BOOLEAN);

BEGIN

  IF (i_fInit = TRUE) THEN BEGIN

    LibSio_Init(TRUE, m_iLOD_BAUDRATE, 'N', 8, 1,
                m_iLOD_TXDLY_CHAR, m_iLOD_TXDLY_LINE);

  END ELSE BEGIN
    LibSio_Init(FALSE, 0, 'N', 0, 0, 0, 0);
  END;
END;
{========================================================= eoF Lod_InitSio() }

{============================================================ Lod_SendFile() }
{ Purpose...: Send LOD file
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Lod_SendFile: BYTE;

  CONST bMODE_TEXT      = 0;
  CONST bMODE_CHECKSUM  = 1;
  CONST bMODE_QUICKLOAD = 2;

  VAR bSio      : BYTE;
  VAR cKbd      : CHAR;
  VAR bCnt      : BYTE;
  VAR bData     : BYTE;
  VAR bWrite    : BYTE;
  VAR fQlWrite  : BOOLEAN;
  VAR bQlRecCnt : BYTE;
  VAR fEof      : BOOLEAN;
  VAR pfEof     : ^BOOLEAN;
  VAR bMode     : BYTE;
  VAR fCtrl     : BOOLEAN;
  VAR fSio      : BOOLEAN;

BEGIN
  bCnt := 0;
  bMode := bMODE_TEXT;
  bSio := g_bLibSioOK;
  fEof := FALSE;
  pfEof := PTR(ADDR(fEof));

  {----------------------------- display OSI hint and wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_TRANSMIT, g_stFile, ' (', File_GetBufBytes, ' bytes)');
  WRITE('In Monitor, type `L`.', g_stMSG_CONTINUE);
  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_TRANSMITTING, g_stFile, g_stMSG_CANCEL);
    LibSio_RxClear;
  END;

  {--------------------------------------------------------------- init SIO }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    Lod_InitSio(TRUE);
  END;

  {---------------------------------------------------------- send the file }
  WHILE ((fEof = FALSE) AND (bSio = g_bLibSioOK)) DO BEGIN
    bData := File_GetByte(pfEof);
    bWrite := 0;
    fSio := TRUE;
    fCtrl := (bData <= ORD(' '));

    IF (fEof = FALSE) THEN BEGIN

      {---------------------------------------------- check for mode change }
      IF (bMode = bMODE_TEXT) THEN BEGIN

        {-------------------------------------------- from text to checksum }
        IF (bData = ORD(';')) THEN BEGIN
          bMode := bMODE_CHECKSUM;
          bCnt := 0;
        END;

        {------------------------------------------- from text to quickload }
        IF (bData = $7F) THEN BEGIN
          bData := File_GetByte(pfEof);
          bMode := bMODE_QUICKLOAD;
          bCnt := 0;
          bQlRecCnt := 0;
        END;
      END;

      {-------------------------------------------- display data on screen }
      CASE bMode OF

        {------------------------------------------------------- text mode }
        bMODE_TEXT: BEGIN

          {------------------------- address will follow, 4 digits and '/' }
          IF (bData = ORD('.')) THEN BEGIN
            bCnt := $80;
            WRITELN;
          END;

          {----------------------- new line after address or 16 data bytes }
          IF ((bCnt = 32) OR (bCnt = $85)) THEN BEGIN
            bCnt := 0;
            WRITELN;
          END;

          {--------------------- in data mode, separate bytes with a space }
          IF (((bCnt AND $81) = $00) AND (bCnt <> 0)) THEN BEGIN
            IF (fCtrl = FALSE) THEN WRITE(' ');
          END;

          {--------------------------------------- check for EOF character }
          IF (bData = $1A) THEN BEGIN
            fEof := TRUE;
          END;

          {------------------------------------- ignore control characters }
          IF ((fCtrl = FALSE) AND (bData <> ORD('/'))) THEN BEGIN
            bCnt := bCnt + 1;

            IF (bData <> $7F) THEN BEGIN          { 7F is ID for quickload }
              bWrite := bData;
            END;
          END;

          {-------------------------------------------- supress line feeds }
          IF (bData = $0A) THEN BEGIN
            fSio := FALSE;
          END;
        END;

        {--------------------------------------------------- checksum mode }
        bMODE_CHECKSUM: BEGIN

          {-------------------------------------------- begin a new record }
          IF (bData = ORD(';')) THEN BEGIN
            bCnt := $80;
            WRITELN;
          END;

          {---- display record size and address separated '<size> <addr> ' }
          IF ((bCnt = $82) OR (bCnt = $86)) THEN BEGIN
            IF (fCtrl = FALSE) THEN WRITE(' ');
            IF (bCnt = $86) THEN bCnt := 0;
          END;

          {------------------------------------- ignore control characters }
          IF ((fCtrl = FALSE) AND (bData <> ORD(';'))) THEN BEGIN
            bWrite := bData;
            bCnt := bCnt + 1;
          END;

          {--------------------------------------- check for EOF character }
          IF (bData = $1A) THEN BEGIN
            fEof := TRUE;
          END;
        END;

        {-------------------------------------------------- quickload mode }
        bMODE_QUICKLOAD: BEGIN
          fQlWrite := TRUE;

          {-------------------------------------------- begin a new record }
          IF ((bData = ORD(';')) AND (bQlRecCnt = 0)) THEN BEGIN
            bQlRecCnt := $80;
            bCnt := $80;
            fQlWrite := FALSE;
            WRITELN;
          END;

          {--------------------------- check for data end (JMP instruction }
          IF ((bData = ORD('$')) AND (bQlRecCnt = 0)) THEN BEGIN
            bQlRecCnt := $40;
            bCnt := $40;
            fQlWrite := FALSE;
            WRITELN;
            WRITE('JMP $');
          END;

          {------------ end of data reached, let jump address pass through }
          IF (bCnt = $43) THEN BEGIN
            bCnt := 0;
            fEof := TRUE;
            fQlWrite := FALSE;
          END;

          {---- display record size and address separated '<size> <addr> ' }
          IF ((bCnt = $82) OR (bCnt = $84)) THEN BEGIN
            WRITE(' ');
            IF (bCnt = $84) THEN bCnt := 0;
          END;

          {--------------------------------- get number of bytes in record }
          IF (bCnt = $81) THEN BEGIN
            bQlRecCnt := (bData + 5);
          END;

          {----------------------------------------- new line after 32 bytes }
          IF (bCnt = 32) THEN BEGIN
            bCnt := 0;
            WRITELN;
            WRITE('        ');
          END;

          {--------------------- update record counter, check for record end }
          IF (bQlRecCnt > 0) THEN BEGIN
            bQlRecCnt := bQlRecCnt - 1;
          END;

          bCnt := bCnt + 1;
        END;
      END;

      {------------------------------------------------ write data to screen }
      IF ((bMode = bMODE_QUICKLOAD) AND (fQlWrite = TRUE)) THEN BEGIN
        WRITE(Lib_CnvBinToHex(bData, 2));
      END;

      IF (bMode <> bMODE_QUICKLOAD) THEN BEGIN
        IF ((bWrite <> 0) AND (bWrite >= ORD(' '))) THEN BEGIN
          WRITE(CHR(bWrite));
        END;
      END;

      {------------------------------------------------- write data to SIO }
      IF (fSio = TRUE) THEN BEGIN
        bSio := LibSio_TxByte(bData);
      END;

      {-------------------------------------------------- apply line delay }
      IF (bData = $0A) THEN BEGIN
        Lib_Delay(m_iLOD_TXDLY_LINE);
      END;
    END;
  END;

  WRITELN;

  IF (bSio = g_bLibSioCANCEL) THEN BEGIN
    WRITELN;
    WRITE('Cancelled. ');
  END;

  Lod_InitSio(FALSE);
  Lod_SendFile := bSio;
END;
{======================================================== eoF Lod_SendFile() }

{========================================================= Lod_ReceiveFile() }
{ Purpose...: Receive LOD file
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Lod_ReceiveFile: BYTE;

  CONST iEND_TIMEOUT : INTEGER = 1500;

  VAR bCnt        : BYTE;
  VAR bSio        : BYTE;
  VAR bSkipSpaces : BYTE;
  VAR cKbd        : CHAR;
  VAR cRx         : CHAR;
  VAR pbRx        : ^BYTE;
  VAR iTimeout    : INTEGER;
  VAR fFirstChar  : BOOLEAN;
  VAR fDone       : BOOLEAN;

BEGIN
  bSio := g_bLibSioOK;
  fFirstChar := TRUE;
  fDone := FALSE;
  bSkipSpaces := 0;
  iTimeout := -1;
  bCnt := 0;
  pbRx := PTR(ADDR(cRx));

  {----------------------------- display OSI hint and wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_RECEIVE, g_stFile, ' (', File_GetBufSize, ' bytes free)');
  WRITE('In Monitor, type `S`<from><to><start(FE00=monitor)><CR>`. ');
  WRITE(g_stMSG_CONTINUE);
  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSio := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_RECEIVING, g_stFile, g_stMSG_CANCEL);
    LibSio_RxClear;
  END;

  {---------------------------------------------------------------- init SIO }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    Lod_InitSio(TRUE);
  END;

  {-------------------------------------------- loop until done or cancelled }
  WHILE ((bSio = g_bLibSioOK) AND (fDone = FALSE)) DO BEGIN
    bSio := LibSio_RxByte(pbRx, iTimeout, TRUE);

    {------------------------------------------------- one byte was received }
    IF ((bSio = g_bLibSioOK) AND (ORD(cRx) <> 0)) THEN BEGIN
      iTimeout := iEND_TIMEOUT;

      {---------------------------------- write received character to buffer }
      IF (File_PutBuffer(cRx) = TRUE) THEN BEGIN

        {------------------------------------------------ convert CR to CRLF }
        IF (cRx = g_cCR) THEN BEGIN
          IF (File_PutBuffer(g_cLF) = FALSE) THEN BEGIN
            bSio := g_bLibSioCANCEL;
          END;
        END;
      END ELSE BEGIN
        bSio := g_bLibSioCANCEL;
      END;

      {-------------------------------------- print received bytes to screen }
      IF (bSio = g_bLibSioOK) THEN BEGIN

        CASE cRx OF
          '.' : BEGIN
                  IF (fFirstChar = TRUE) THEN BEGIN
                    fFirstChar := FALSE;
                    bSkipSpaces := 2;
                    bCnt := $FE;
                  END ELSE BEGIN
                    WRITELN;
                    bSkipSpaces := 3;
                  END;
                  WRITE(cRx);
                END;
          '/' : WRITELN;
          'G' : WRITE(cRx);
        END;

        IF (Lib_IsHexChar(cRx)) THEN BEGIN
          WRITE(cRx);

          IF (bSkipSpaces <> 0) THEN BEGIN
            bSkipSpaces := bSkipSpaces - 1;
          END ELSE BEGIN
            IF (ODD(bCnt)) THEN WRITE(' ');
            bCnt := bCnt + 1;
          END;

          IF (bCnt = 32) THEN BEGIN
            bCnt := 0;
            WRITELN;
          END;
        END;
      END;
    END;

    IF ((iTimeout <> -1) AND (bSio = g_bLibSioTIMEOUT)) THEN BEGIN
      bSio := g_bLibSioOK;
      fDone := TRUE;
    END;
  END;

  {----------------------------------------------------- save buffer to disk }
  IF (bSio = g_bLibSioOK) THEN BEGIN
    File_WriteBuffer;
  END;

  {--------------------------------------------------------------- cancelled }
  IF (bSio = g_bLibSioCANCEL) THEN BEGIN
    WRITELN;
    WRITE('Cancelled. Write buffer to ', g_stFile, ' ? (Y/N) ');
    cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));

    IF (cKbd = 'Y') THEN BEGIN
      File_WriteBuffer;
    END;
  END;

  Lod_InitSio(FALSE);
  Lod_ReceiveFile := bSio;
END;
{===================================================== eoF Lod_ReceiveFile() }

{############################################################ EOF OSILOD.INC }